local AceAddon = AceLibrary("AceAddon-2.0")
local Tablet = AceLibrary("Tablet-2.0")
local Tooltip = AceLibrary("Gratuity-2.0")

local L = AceLibrary("AceLocale-2.2"):new("Cartographer_Vendors")
L:RegisterTranslations("enUS", function() return {
	["Filter"] = true,
	["Filter out vendors"] = true,
	
  ["Vendors"] = true,
  ["Module description"] = "Vendor Mapnotes for Cartographer",
  
	["Select all"] = true,
	["Select none"] = true,

  ["Vendor"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Filter"] = "필터",
	["Filter out vendors"] = "상인 아웃 필터",

  ["Vendors"] = "상인",
  ["Module description"] = "맵노트에 상인을 표시합니다.",
	
	["Select all"] = "모두 선택",
	["Select none"] = "선택 없음",

  ["Vendor"] = "상인",
} end)

L:RegisterTranslations("deDE", function() return {
	["Filter"] = "Filter",
	["Filter out vendors"] = "H\195\164ndler filtern",
	
  ["Vendors"] = "H\195\164ndler",
  ["Module description"] = "H\195\164ndler-Kartennotizen f\195\88r Cartographer",
  
	["Select all"] = "Alles ausw\195\164hlen",
	["Select none"] = "Nichts ausw\195\164hlen",
    
  ["Vendor"] = "H\195\164ndler",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Filter"] = "篩檢",
	["Filter out vendors"] = "篩檢商人",
	
  ["Vendors"] = "商人",
  ["Module description"] = "Cartographer 商人地圖註記",
  
	["Select all"] = "全部選擇",
	["Select none"] = "全部取消",
    
  ["Vendor"] = "商人",
} end)

L:RegisterTranslations("esES", function() return {
	["Filter"] = "Filtrar",
	["Filter out vendors"] = "Filtra vendedores",
	
  ["Vendors"] = "Vendedores",
  ["Module description"] = "Notas de mapa de vendedores para Cartographer",
  
	["Select all"] = "Seleccionar todo",
	["Select none"] = "No seleccionar nada",

  ["Vendor"] = "Vendedor",
} end)

local mod
if AceLibrary:HasInstance("AceDebug-2.0") then 
    mod = Cartographer:NewModule("Vendors", "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceDebug-2.0")
    mod:SetDebugging(false)
else
    mod = Cartographer:NewModule("Vendors", "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
    -- mod.Debug = function() return end
end

mod:RegisterDB("Cartographer_VendorsDB")

mod.icon = {
	["Vendor"] = {
		text = L["Vendor"],
		path = "Interface\\Icons\\INV_Misc_ElvenCoins",
		width = 12,
		height = 12
	},
}

mod.vendors = {}

function mod:OnInitialize()
	self:RegisterDefaults("profile", {
		filter = {
			['*'] = true,
		}
	})

	local aceopts = {}
	aceopts.toggle = {
		name = AceLibrary("AceLocale-2.2"):new("Cartographer")["Enabled"],
		desc = AceLibrary("AceLocale-2.2"):new("Cartographer")["Suspend/resume this module."],
		type  = 'toggle',
		order = -1,
		get   = function() return Cartographer:IsModuleActive(self) end,
		set   = function() Cartographer:ToggleModuleActive(self) end,
	}

    --[[
	aceopts.filter = {
		name = L["Filter"],
		desc = L["Filter out quests"],
		type = 'group',
		args = {
			all = {
				name = L["Select all"],
				desc = L["Select all"],
				type = 'execute',
				func = function()
					for k in pairs(self.icon) do
						self:ToggleShowingQuests(k, true)
					end
				end,
				order = 1,
			},
			none = {
				name = L["Select none"],
				desc = L["Select none"],
				type = 'execute',
				func = function()
					for k in pairs(self.icon) do
						self:ToggleShowingQuests(k, false)
					end
				end,
				order = 2,
			},
			blank = {
				type = 'header',
				order = 3,
			}
		}
	}
	for k,v in pairs(self.icon) do
		local k = k
		aceopts.filter.args[k] = {
			name = v.text,
			desc = v.text,
			type = 'toggle',
			get = function()
				return self:IsShowingQuests(k)
			end,
			set = function(value)
				return self:ToggleShowingQuests(k, value)
			end,
		}
	end
    --]]
    
	Cartographer.options.args["Vendors"] = {
		name = L["Vendors"],
		desc = L["Module description"],
		type = 'group',
		args = aceopts,
		handler = self,
	}
	-- AceLibrary("AceConsole-2.0"):InjectAceOptionsTable(self, Cartographer.options.args["Vendors"])

    if (not self.db.faction.notes) then self.db.faction.notes = {} end
    
    local oldDB = Cartographer:AcquireDBNamespace("Vendors")
    if (oldDB.faction.notes) then
        for k, v in pairs(oldDB.faction.notes) do
            if (k ~= "version") and (v ~= nil) then
                if (not self.db.faction.notes[k]) then self.db.faction.notes[k] = {} end
                for vk, vv in pairs(v) do
                    if (vv ~= nil) then
                        if (not self.db.faction.notes[k][vk]) then
                            -- self.db.faction.notes[k][vk] = {}
                            self.db.faction.notes[k][vk] = vv
                        end
                    end
                end
            end
        end
    end
    oldDB = nil    
end

function mod:OnEnable()
	if Cartographer_Notes then
		if not self.iconsregistered then
			for k,v in pairs(self.icon) do
				Cartographer_Notes:RegisterIcon(k, v)
			end
			self.iconsregistered = true
		end

		Cartographer_Notes:RegisterNotesDatabase('Vendors', self.db.faction.notes, self)
	else
		Cartographer:ToggleModuleActive(self, false)
	end

    self:RegisterEvent("MERCHANT_SHOW", function() self:RegisterVendor(UnitName("NPC")) end, 2)
end

function mod:OnDisable()
	self:UnregisterAllEvents()

	if Cartographer_Notes then
		Cartographer_Notes:UnregisterNotesDatabase('Vendors')
	end
end

function mod:SetNote(vendor)
	local x, y = GetPlayerMapPosition("player")
	if x == 0 and y == 0 then return end
	local zone = GetRealZoneText()
    
    local note = {}
    note.icon = L["Vendor"]
    note.title = vendor.name
    if (vendor.type) then note.info = vendor.type
    else note.info = L["Vendor"] end
    
	for zone, x_p, y_p, icon, creator, data in Cartographer_Notes:IterateNearbyNotes(zone, x, y, 5, "Vendors", nil, true) do
		if data.title == note.title then
			Cartographer_Notes:DeleteNote(zone, x_p, y_p)
		end
	end
	Cartographer_Notes:SetNote(zone, x, y, note.icon, "Vendors", note)
end

function mod:RegisterVendor(vendor)
    if (not vendor) then return end
    if (type(vendor) ~= "string") then return end
    
    if (not mod.vendors) then mod.vendors = {} end
    if (not mod.vendors[vendor]) then 
        mod.vendors[vendor] = {} 
        mod.vendors[vendor].name = vendor
    end
    
    if (not mod.vendors[vendor].type) then
        Tooltip:SetUnit("NPC")
        local result = Tooltip:GetLine(2)
        if (result) then mod.vendors[vendor].type = result end
    end
    
    self:SetNote(mod.vendors[vendor])
end

function mod:IsNoteHidden(zone, id, vendortype)
	return not self.db.profile.filter[vendortype]
end

function mod:IsShowingVendors(vendortype)
	return self.db.profile.filter[vendortype]
end

function mod:ToggleShowingVendors(vendortype, value)
	if value == nil then
		value = not self.db.profile.filter[vendortype]
	end
	self.db.profile.filter[vendortype] = value

	Cartographer_Notes:RefreshMap()
end
